/* 
    ProMoIJ_M2_Registration is an ImageJ macro developed to Register images from 
    two-photon life imaging experiments.
    
    Copyright (C) 2017  Jorge Valero Gómez-Lobo, Iñaki Paris & Amanda Sierra.

    ProMoIJ_M2_Registration is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    ProMoIJ_M2_Registration is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

//This macro has been developed by Iñaki Paris, Amanda Sierra & Jorge Valero (jorge.valero@achucarro.org). 
//If you have any doubt about how to use it, please contact me.

//Optimization general variable
var Optim="";
if (isOpen("Log")){
	selectWindow("Log");
	run("Close");
}
//License
Dialog.create("GNU GPL License");
Dialog.addMessage("ProMoIJ_M2_Registration (C) 2017 J. Valero, I. Paris & A. Sierra.");
Dialog.setInsets(10, 20, 0);
Dialog.addMessage("ProMoIJ_M2_Registration  comes with ABSOLUTELY NO WARRANTY.");
Dialog.setInsets(0, 20, 0);
Dialog.addMessage("This is free software, and you are welcome to redistribute it under certain conditions. Click on the Help button for details.");
Dialog.addHelp("http://www.gnu.org/licenses/gpl.html");
Dialog.show();

//Selects original image files to be registered. Image files will be removed from the original folder, transfered as processed images to second folder, and then saved as registered originalname_Reg.tif files in a final third folder.
dir=getDirectory("Please, select the M2 input images folder: Original or M1 Cropped");
dirreg=getDirectory("Please, select the M2 output folder: M2 Registered");

//Opens an image file and obtains the image dimensions
setBatchMode(true);
images=getFileList(dir);
v=0;
while(endsWith(images[v], "/")) v++;
run("Bio-Formats Importer", "open=["+dir+images[v]+"] color_mode=Grayscale open_files view=Hyperstack stack_order=XYCZT");
getDimensions(width0, height0, channels0, slices0, frames0);
selectWindow(getTitle());
close();
chan0=newArray(channels0);
for (i=1; i<=channels0; i++) chan0[i-1]=i;


/*The user provides the value of the following parameters:
 * M2.1 Activate Batch mode: Activates batch processing of images (without visualization)
 * M2.2 Apply 3D Drift Correction: Activates 3D registration with the plugin “Correct 3D drift
 * M2.3 Apply MultistackReg: Activates registration with the plugin “MultiStackReg"
 * M2.4 Registering algorithm: Selects the algorithm used during registration
 */
items=newArray("Rigid Body", "Translation", "Affine", "Scaled Rotation");
items2=newArray("Previous time point", "First time point");
correction3D=newArray("Before", "After", "Before and After", "None");
Dialog.create("ProMoIJ Macro 2 Registration parameters");

Dialog.addChoice("M2.1 Apply 3D Drift Correction", correction3D);
Dialog.addCheckbox("M2.1.1 Fine 3D Drift Correction Before", false);
Dialog.addCheckbox("M2.1.2 Fine 3D Drift Correction After", true);
Dialog.addCheckbox("M2.2 Apply MultistackReg", true);
Dialog.addChoice("M2.3 Registering algorithm", items);
Dialog.addNumber("M2.4 Number of MultistackReg iterations", 1);
Dialog.addChoice("M2.5 Reference time point or MultistackReg", items2);
Dialog.addCheckbox("M2.6 Optimization mode", false);
Dialog.addHelp("http://www.achucarro.org/downloads");
Dialog.show()

corr3D=Dialog.getChoice();
FineRegB=Dialog.getCheckbox();
FineRegA=Dialog.getCheckbox();
Multi=Dialog.getCheckbox();
typeReg=Dialog.getChoice();
iter=Dialog.getNumber();
if (iter==0) iter=1;
regmode=Dialog.getChoice();
ver=Dialog.getCheckbox();


if (Multi==true) print("Registering algorithm choice: "+typeReg+" with "+iter+" iterations");

//Loop that reads all image files within original folder and processes them. Only image files, not folders, will be analyzed.
for (v=0; v<images.length; v++){
	print("Processing image file "+v+1+" of "+images.length);
	if (endsWith(images[v], "/")) print("NO"); 
	else work(dir+images[v]);
	for (gc=0; gc<5; gc++) run("Collect Garbage");
	
	//Optimization paramaters dialog.
	if (ver==true){
			if (Optim=="Change parameters"){
			items=newArray("Rigid Body", "Translation", "Affine", "Scaled Rotation");
			items2=newArray("Previous time point", "First time point");
			correction3D=newArray("Before", "After", "Before and After", "None");
			Dialog.create("ProMoIJ Macro 2 Registration parameters");
			
			
			Dialog.addChoice("M2.1 Apply 3D Drift Correction", correction3D, corr3D);
			Dialog.addCheckbox("M2.1.1 Fine 3D Drift Correction Before", FineRegB);
			Dialog.addCheckbox("M2.1.2 Fine 3D Drift Correction After", FineRegA);
			Dialog.addCheckbox("M2.2 Apply MultistackReg", Multi);
			Dialog.addChoice("M2.3 Registering algorithm", items, typeReg);
			Dialog.addNumber("M2.4 Number of MultistackReg iterations", iter);
			Dialog.addChoice("M2.5 Reference time point or MultistackReg", items2, regmode);
			Dialog.addCheckbox("M2.6 Optimization mode", true);
			Dialog.addHelp("http://www.achucarro.org/downloads");
			Dialog.show()
			
			corr3D=Dialog.getChoice();
			FineRegB=Dialog.getCheckbox();
			FineRegA=Dialog.getCheckbox();
			Multi=Dialog.getCheckbox();
			typeReg=Dialog.getChoice();
			iter=Dialog.getNumber();
			if (iter==0) iter=1;
			regmode=Dialog.getChoice();
			ver=Dialog.getCheckbox();
			v--;
		}
		if (Optim=="Register all images with the current settings"){
			v=-1;
			ver=false; 
		}
	}
		
}
print("I am done, have fun with your images");

//The function "work" opens the image files, register them, and save them in the final folder.
function work(imagepath){
     	run("Bio-Formats Importer", "open=["+imagepath+"] color_mode=Grayscale open_files view=Hyperstack stack_order=XYCZT");
		getDimensions(width1, height1, channels1, slices1, frames1);
		name=getTitle();
		imagename=images[v];
		if (slices0>1 && (corr3D=="Before" || corr3D=="Before and After")){
			correction3Ddrift(FineRegB);
			getDimensions(width1, height1, channels1, slices1, frames1);
		}
		if (Multi==true) multistackReg();
		if (slices0>1 && (corr3D=="After" || corr3D=="Before and After")){
			correction3Ddrift(FineRegA);
			getDimensions(width1, height1, channels1, slices1, frames1);
		}
			if (corr3D!="None") blackCropper();
			selectWindow(imagename);	
			saveAs("Tiff", dirreg+imagename+"_Reg.tif");
			selectWindow(imagename+"_Reg.tif");
			close();

			//Optimization mode
			if (ver==true){
				setBatchMode(false);
				run ("Close All");
				run("Bio-Formats Importer", "open=["+imagepath+"] color_mode=Grayscale open_files view=Hyperstack stack_order=XYCZT");
				rename("Original");
				selectWindow("Original");
				run("Z Project...", "projection=[Max Intensity] all");
				rename("MAX_Original");
				run("Maximize");
				selectWindow("Original");
				//close();
				run("Bio-Formats Importer", "open=["+dirreg+imagename+"_Reg.tif] color_mode=Grayscale open_files view=Hyperstack stack_order=XYCZT");
				rename("Registered");
				selectWindow("Registered");
				run("Z Project...", "projection=[Max Intensity] all");
				rename("MAX_Registered");
				run("Maximize");
				selectWindow("Registered");
				//close();
				run("Tile");
				waitForUser("Optimization", "Please, check the images and click OK.\n   \n                     Zoom in: Ctrl+\n                     Zoom out: Ctrl-");
				//Optimization dialog
				Dialog.create("Optimization mode");
				items=newArray("Check next image", "Change parameters", "Register all images with the current settings");
				Dialog.addRadioButtonGroup("Select one option" , items, 3, 1, "Check next image");
				Dialog.addHelp("http://www.achucarro.org/downloads");
				Dialog.show;
				Optim=Dialog.getRadioButton();
				run ("Close All");
				setBatchMode(true);
			}
}

//The function "correction3Ddrift" registers the image using the "Correct 3D Drift" ImageJ plugin
function correction3Ddrift(Fine3DCorr){
			selectWindow(imagename);
			print("Correcting 3D drift");
			if (Fine3DCorr==true) run("Correct 3D drift", "channel=1 multi_time_scale sub_pixel only=0 lowest=1 highest="+slices1);
			else run("Correct 3D drift", "channel=1");
			//run("Correct 3D drift", "channel=1 multi_time_scale sub_pixel edge_enhance only=0 lowest=1 highest="+slices1);
			selectWindow(imagename);
			close();
			selectWindow("registered time points");
			rename(imagename);
}

//The function "multistackReg" registers the image using the "MultiStackReg" ImageJ plugin
function multistackReg(){
		selectWindow(imagename);
		getDimensions(x, y, c, z, t);
		align="image1=align1 ";
		
		//Applies MultiStackReg plugin to register the z-stack over time. It creates duplicates of the z-stacks at t=i and and t=i+1 and aligns them in a concatenate loop over the time course.
		if (slices0>1) run("Duplicate...", "title=align1 duplicate frames=1");
		else {
			setSlice(1);
			run("Duplicate...", "title=align1");
		}
		for (iii=0; iii<iter; iii++){
			for (ii=1; ii<t; ii++){
				print("Running MultiStackReg "+ii+" of "+t+ " (iteration "+iii+1+" of "+iter+")");
				selectWindow(imagename);
				if (slices0>1) run("Duplicate...", "title=align"+(ii+1)+" duplicate frames="+(ii+1));
				else {
					setSlice(ii+1);
					run("Duplicate...", "title=align"+(ii+1));
				}
				if (regmode=="First time point") selectWindow("align1");
				else selectWindow("align"+ii);
				setSlice(1);
				
				if (regmode=="First time point") run("MultiStackReg", "stack_1=align1 action_1=[Use as Reference] file_1=[] stack_2=align"+ii+1+" action_2=[Align to First Stack] file_2=[] transformation=["+typeReg+"]");
				else run("MultiStackReg", "stack_1=align"+ii+" action_1=[Use as Reference] file_1=[] stack_2=align"+ii+1+" action_2=[Align to First Stack] file_2=[] transformation=["+typeReg+"]");
				//Obtains the number of aligned images
				align=align+"image"+(ii+1)+"="+"align"+(ii+1)+" ";
				for (gc=0; gc<5; gc++) run("Collect Garbage");
			}
			//aligns last and first frames to perform extra iterations
			if (iter>1) run("MultiStackReg", "stack_1=align"+t+" action_1=[Use as Reference] file_1=[] stack_2=align"+1+" action_2=[Align to First Stack] file_2=[] transformation=["+typeReg+"]");	
		}
		
			
		selectWindow(imagename);
		close();

		
			//Merges aligned z-stacks into a single hyperstack
			run("Concatenate...", "  title=[Concatenated Stacks] open "+align);
			if (slices0>1) run("Stack to Hyperstack...", "order=xyczt(default) channels=1 slices="+z+" frames="+t+" display=Color");
			selectWindow("Concatenated Stacks");
			rename(imagename);
}


function blackCropper(){
	selectWindow(imagename);
	getDimensions(width, height, channels, slices, frames);
	maxslice=slices;
	minslice=1;
	run("Duplicate...", "title=Frame1 duplicate frames=1");
	selectWindow(name);
	run("Duplicate...", "title=FrameFin duplicate frames="+frames);
	selectWindow("Frame1");
	//print(slices, frames);
	for (i=slices; i>0; i--){
		setSlice(i);
		getStatistics(area, mean, min, max, std, histogram);
		//print(max);
		if (max>0){
			maxslice=i;
			//print(mean, maxslice);
			i=0;
		}
	}
	selectWindow("Frame1");
	for (i=1; i<=slices; i++){
		setSlice(i);
		getStatistics(area, mean, min, max, std, histogram);
		if (max>0){
			minslice=i;
			i=slices+1;
		}
	}
	selectWindow("Frame1");
	close();
	selectWindow("FrameFin");
	for (i=slices; i>0; i--){
		setSlice(i);
		getStatistics(area, mean, min, max, std, histogram);
		if (max>0){
			if (i<maxslice)maxslice=i;
			i=0;
		}
	}
	selectWindow("FrameFin");
	for (i=1; i<=slices; i++){
		setSlice(i);
		getStatistics(area, mean, min, max, std, histogram);
		if (max>0){
			if (i>minslice) minslice=i;
			i=slices+1;
		}
	}
	selectWindow("FrameFin");
	close();
	//print (minslice,maxslice);
	selectWindow(imagename);
	run("Duplicate...", "title=temporal duplicate slices="+minslice+"-"+maxslice+"");
	selectWindow(imagename);
	close();
	selectWindow("temporal");
	rename(imagename);
}




