/*  ProMoIJ_M4_Process_selection is an ImageJ macro developed to design ROIs and reorientation lines 
 *  to run the macro ProMoIJ in image files from two-photon life imaging experiments
    
    Copyright (C) 2017  Jorge Valero Gómez-Lobo, Iñaki Paris & Amanda Sierra.

  ProMoIJ_M4_Process_selection is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

     ProMoIJ_M4_Process_selection is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

//This macro has been developed by Iñaki Paris, Amanda Sierra & Jorge Valero (jorge.valero@achucarro.org). 
//If you have any doubt about how to use it, please contact me.


//License
Dialog.create("GNU GPL License");
Dialog.addMessage("ProMoIJ_M4_Process_selection (C) 2017 J. Valero, I. Paris & A. Sierra.");
Dialog.setInsets(10, 20, 0);
Dialog.addMessage("ProMoIJ_M4_Process_selection comes with ABSOLUTELY NO WARRANTY.");
Dialog.setInsets(0, 20, 0);
Dialog.addMessage("This is free software, and you are welcome to redistribute it under certain conditions. Click on the Help button for details.");
Dialog.addHelp("http://www.gnu.org/licenses/gpl.html");
Dialog.show();

var Optim="";
var thrs=0;
if (isOpen("Log")){
	selectWindow("Log");
	run("Close");
}

/*SelectS the folder that contains the original image files to be registered. Image files will be removed from the original folder 
 * and transfered as processed images to a second folder. 
 *If the user does not want to use a particular image file, it will be moved to a third folder for non-processed image files. CAMBIAR COMO EN M1 CROPPING
 *Finally, the ROI file containing the ROIs and reorientation lines corresponding to a single cell, 
 *will be stored as cellnumber+originalname without extension.zip in a fourth folder
 */
dir=getDirectory("Please, select the M4 input images folder: M3 Corrected");
dirP=getDirectory("Please, select the M4 output folder for processed images: M4 Processed");
dirNP=getDirectory("Please, select the M4  output folder for discarded images: M4 Discarded");
dirRoi=getDirectory("Please, select the M4 output folder for ROIs: M4 Selection ROIs");

Dialog.create("ProMoIJ Macro 4 Process selection Parameters");
Dialog.addNumber("M4.1 Threshold level", 0);
Dialog.addCheckbox("M4.2 Optimization mode", false);
Dialog.addHelp("http://www.achucarro.org/downloads");
Dialog.show();

thrs=Dialog.getNumber();
ver=Dialog.getCheckbox();


//Loop that reads all image files within the original folder and allows selecting processes and drawing of orientation vectors.
fotos=getFileList(dir);
for (fot=0; fot<fotos.length; fot++){
	work();
	if (ver==true){
		if (Optim=="Change Threshold") fot--;
		if (Optim=="Select processes using the current Threshold"){
			proceed=getBoolean("All images will be processed using the folowing Threshold level: "+thrs+".\nPlease, take note of this threshold level for further analysis.\nDo you want to proceed?");
			if (proceed==true){
				fot=-1;
				ver=false; 
			}
			else fot--;
		}
	}
}

var slices=1;

//The function "work" opens the image files, makes a maximum intensity projection and then a time color-coded projection. Finally, it will allow the user to select processes and draw orientation vectors. 
function work(){
	run("Bio-Formats Importer", "open=["+dir+fotos[fot]+"] autoscale color_mode=Grayscale open_files view=Hyperstack stack_order=XYCZT"); 
	name=getTitle();
	namenoext=File.nameWithoutExtension();
	run("Maximize");
	if (ver==true && Optim!="Check next image"){
 		run("Threshold...");
 		waitForUser("Set Threshold", "Please, set Threshold level using the upper sliding bar of the Threshold tool of the Image menu.\nThen click OK. \n \nCAUTION: DO NOT click on the Apply Button of the Threshold tool!!!.\n   \nPlease, check Threshold levels navigating through slices and frames using the sliding bars at the bottom of the image.\n   \n                     Zoom in: Ctrl+\n                     Zoom out: Ctrl-");
  		selectImage(name); 
 	 	getThreshold(thrs, upper);
	}
	run("Duplicate...", "title=AND duplicate");
	getDimensions(width1, height1, channels1, slices1, frames1);
	max=0;
	for (i=1; i<=slices1; i++){
		setSlice(i);
		getStatistics(area, mean, min, maxtemp, std, histogram);
		if (maxtemp>max) max=maxtemp;
	}
	setThreshold(thrs, max);
	setOption("BlackBackground", true);
	run("Convert to Mask", "method=Default background=Dark black");
	framecalculator("AND", "AND_frames");
	selectWindow("AND");
	close();
	selectWindow("AND_frames");
	run("Red");
	run("RGB Color");
	
	if (slices<2){
		selectWindow(name);
		run("Duplicate...", "title=Temporal duplicate");
		selectWindow("Temporal");
		run("RGB Color");
		imageCalculator("Add create stack", "Temporal","AND_frames");
		selectWindow("Result of Temporal");
		rename("MAX_FRAMES");
		selectWindow("Temporal");
		close();
		selectWindow("AND_frames");
	}
	else{
		run("Z Project...", "projection=[Max Intensity] all");
		selectWindow("AND_frames");
		close();
		selectWindow(name);
		run("Z Project...", "projection=[Max Intensity] all");
		run("RGB Color");
		imageCalculator("Add create stack", "MAX_"+name,"MAX_AND_frames");
		selectWindow("Result of MAX_"+name);
		rename("FRAMES_"+name);
		run("Maximize");
		selectWindow("MAX_"+name);
		close();
		selectWindow("FRAMES_"+name);
		run("Z Project...", "projection=[Max Intensity] all");
		rename("MAX_FRAMES");
		run("Maximize");
		selectWindow("MAX_AND_frames");
	 	}
		close();

	if (ver==true){
		run("Tile");
		waitForUser("Optimization", "Please, check the images and click OK. \nCurrent image Threshold: "+thrs+"\n   \n                     Zoom in: Ctrl+\n                     Zoom out: Ctrl-");
		//Optimization dialog
		Dialog.create("Optimization mode");
		items=newArray("Check next image", "Change Threshold", "Select processes using the current Threshold");
		Dialog.addRadioButtonGroup("Select one option" , items, 3, 1, "Check next image");
		Dialog.addHelp("http://www.achucarro.org/downloads");
		Dialog.show;
		Optim=Dialog.getRadioButton();
		run ("Close All");
	}
	if (ver==false){
		//Allows the user to check the image, decide whether it will be used, indicate the number of cells to be analyzed, and draw the ROIs (selected processes) and orientation vectors
		run("Tile");
		waitForUser("Please, check the image.");
		usar=getBoolean("Do you want to select any process in this image?");
	
		if (usar==true){
			c=0;
			do {
				cont=true;
				roihechoant=roiManager("count");
				if (roihechoant>0){
					roiManager("Deselect");
					roiManager("Delete");
				}
				//This loop will finish when the user decides not to select any further processes from the current cell to be analyzed 
				do{
					roihechoant=roiManager("count");
					//This loop allows the user to draw a ROI and add it to the ROI Manager. The program will not continue until at least one ROI is added to the ROI manager.
					do{
						//run("Tile");
						setTool("polygon");
						waitForUser("- Please select a single, non-ramified process from a main branch from cell number "+c+" by drawing a ROI around it. \n- Add it to the ROI Manager pressing the letter \"t\". \n- Then, press OK.");
						roihecho=roiManager("count");
						if (roihecho!=roihechoant+1) waitForUser("Please, add one process selection to the ROI Manager");
					} while (roihecho!=roihechoant+1);
					//roiManager("Select", roihecho-1);
					//run("Duplicate...", "title=crop duplicate");
					setTool("arrow");
					run("Arrow Tool...", "width=1 size=2 color=Yellow style=Filled");
					run("Select None");
					roihechoant=roiManager("count");
					
					//This loop allows the user to draw an orientation vector and add it to the ROI Manager. The program will not continue until at least one orientating vector per process is added to the ROI manager.
					do{	
						roiManager("Show All");
						waitForUser("- Please, draw an ARROW along the major axis of the process STARTING basal constant region and pointing to the tip. \n- Add it to the ROI Manager pressing the letter \"t\". \n- Then, press OK.");
						roihecho=roiManager("count");
						roiManager("Select", roihecho-1);
						typeSel=selectionType();
						if (roihecho==roihechoant+1){
							if (typeSel!=5){
							roiManager("Select", roihecho-1);
							roiManager("Delete");
							roihecho=roiManager("count");
							setTool("arrow");
							}
						}
						if (roihecho!=roihechoant+1) waitForUser("Please, add one arrow to the ROI Manager"); 
					} while (roihecho!=roihechoant+1);
					
					//selectWindow("crop");
					//close();
					roiManager("Show All");
					cont=getBoolean("Do you want to select another process from the same cell?");
				}while (cont==true);
				
				//Saves the ROIs from the current cell and cleans the ROI Manager
				c++;
				roiManager("Deselect");
				roiManager("Save", dirRoi+c+namenoext+".zip");
				roiManager("Delete");
				anothercell=getBoolean("Do you want to select processes from another cell in this image?");
			}while (anothercell==true);
			File.rename(dir+fotos[fot], dirP+fotos[fot]);
		}
		//If the user decides not to use the current image file it will be moved to a third folder for non-processed image files
		else File.rename(dir+fotos[fot], dirNP+fotos[fot]);
		if (slices>1) {
			//selectWindow("MAX_SLICES_"+name);
			//close();
			selectWindow("FRAMES_"+name);
			close();
			selectWindow("MAX_FRAMES");
			close();
		}
		else {
			selectWindow("SLICES_"+name);
			close();
		}
		selectWindow(name);
		close();		
		}
	
}

//The function "framecalculator" performs logical operations with two images.
function framecalculator(option, imagetitle){
	getDimensions(width, height, channels, slices, frames);
	run("Duplicate...", "title=ole duplicate");
	for (i=2; i<=frames; i++){
		if (i==2){
			if (slices>1) run("Duplicate...", "title=["+imagetitle+"] duplicate frames=1");
			else {
				setSlice(1);
				run("Duplicate...", "title=["+imagetitle+"]");
			}
			setBatchMode(true);
		}
		selectWindow("ole");
		if (slices>1) run("Duplicate...", "title=ole-"+i+" duplicate frames="+i);
		else {
				setSlice(i);
				run("Duplicate...", "title=ole-"+i);
			}
		imageCalculator(option+" stack", imagetitle,"ole-"+i);
		selectWindow("ole-"+i);
		close();
	}
	selectWindow("ole");
	close();
	setBatchMode(false);
}


